package ru.dbotthepony.mc.otm.screen;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.Font;
import net.minecraft.client.gui.screens.inventory.AbstractContainerScreen;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.network.chat.Component;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.inventory.Slot;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.menu.MatteryMenu;
import ru.dbotthepony.mc.otm.menu.slot.BatterySlot;
import ru.dbotthepony.mc.otm.menu.slot.MatterySlot;
import ru.dbotthepony.mc.otm.menu.slot.MachineOutputSlot;
import ru.dbotthepony.mc.otm.menu.slot.MatterContainerInputSlot;
import ru.dbotthepony.mc.otm.menu.widget.AbstractWidget;
import ru.dbotthepony.mc.otm.screen.panels.EditablePanel;

import java.util.ArrayList;

public class MatteryScreen<T extends MatteryMenu> extends AbstractContainerScreen<T> {
	protected static final ResourceLocation CONTAINER_BASE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/generic_machine.png");
	public static final ResourceLocation WIDGETS = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/widgets.png");

	protected ResourceLocation CONTAINER_BACKGROUND() {
		return CONTAINER_BASE;
	}

	protected ArrayList<EditablePanel> panels = new ArrayList<>();

	public MatteryScreen(T p_97741_, Inventory p_97742_, Component p_97743_) {
		super(p_97741_, p_97742_, p_97743_);

		imageWidth = 176;
		imageHeight = 178;
		titleLabelY = 4;

		inventoryLabelY = imageHeight - 92;
	}

	@Override
	public boolean mouseClicked(double p_97748_, double p_97749_, int p_97750_) {
		for (var panel : panels) {
			if (panel.mouseClickedChecked(p_97748_, p_97749_, p_97750_)) {
				return true;
			}
		}

		return super.mouseClicked(p_97748_, p_97749_, p_97750_);
	}

	@Override
	public boolean mouseDragged(double p_97752_, double p_97753_, int p_97754_, double p_97755_, double p_97756_) {
		for (var panel : panels) {
			if (panel.mouseDraggedChecked(p_97752_, p_97753_, p_97754_, p_97755_, p_97756_)) {
				return true;
			}
		}

		return super.mouseDragged(p_97752_, p_97753_, p_97754_, p_97755_, p_97756_);
	}

	@Override
	public boolean mouseReleased(double p_97812_, double p_97813_, int p_97814_) {
		for (var panel : panels) {
			if (panel.mouseReleasedChecked(p_97812_, p_97813_, p_97814_)) {
				return true;
			}
		}

		return super.mouseReleased(p_97812_, p_97813_, p_97814_);
	}

	@Override
	public boolean mouseScrolled(double p_94686_, double p_94687_, double p_94688_) {
		for (var panel : panels) {
			if (panel.mouseScrolledChecked(p_94686_, p_94687_, p_94688_)) {
				return true;
			}
		}

		return super.mouseScrolled(p_94686_, p_94687_, p_94688_);
	}

	@Override
	public boolean keyPressed(int p_97765_, int p_97766_, int p_97767_) {
		for (var panel : panels) {
			if (panel.keyPressed(p_97765_, p_97766_, p_97767_)) {
				return true;
			}
		}

		return super.keyPressed(p_97765_, p_97766_, p_97767_);
	}

	@Override
	protected void renderTooltip(PoseStack pose, int mouseX, int mouseY) {
		super.renderTooltip(pose, mouseX, mouseY);

		if (this.hoveredSlot == null && menu.getCarried().isEmpty()) {
			for (AbstractWidget widget : menu.widget_list) {
				if (widget.renderTooltip(this, pose, leftPos, topPos, mouseX, mouseY)) {
					return;
				}
			}
		}
	}

	@Override
	public void render(PoseStack pose, int mouseX, int mouseY, float p_98421_) {
		for (AbstractWidget<?> widget : menu.widget_list) {
			widget.updateIsHovered(leftPos, topPos, mouseX, mouseY);
		}

		this.renderBackground(pose);
		super.render(pose, mouseX, mouseY, p_98421_);
		this.renderForeground(pose, mouseX, mouseY, p_98421_);
		this.renderTooltip(pose, mouseX, mouseY);
	}

	protected void renderRegularSlot(PoseStack pose, int x, int y) {
		this.blit(pose, leftPos + x - 1, topPos + y - 1, 0, 96, 18, 18);
	}

	protected void renderOutputSlot(PoseStack pose, int x, int y) {
		this.blit(pose, leftPos + x - 5, topPos + y - 5, 18, 96, 26, 26);
	}

	protected void renderBatterySlot(PoseStack pose, int x, int y) {
		renderRegularSlot(pose, x, y);
		this.blit(pose, leftPos + x - 1, topPos + y - 1, 0, 114, 18, 18);
	}

	protected void renderForeground(PoseStack stack, int mouseX, int mouseY, float p_98421_) {

	}

	public Font getFont() {
		return font;
	}

	@Override
	protected void renderBg(PoseStack pose, float p_97788_, int mouseX, int mouseY) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, CONTAINER_BACKGROUND());
		// PoseStack, x, y, image_start_x, image_start_y, rect_size_x, rect_size_y
		this.blit(pose, leftPos, topPos, 0, 0, this.imageWidth, this.imageHeight);

		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, WIDGETS);

		for (Slot slot : menu.slots) {
			if (slot instanceof BatterySlot slot1 && slot1.auto_bg) {
				renderBatterySlot(pose, slot1.x, slot1.y);
			} else if (slot instanceof MachineOutputSlot slot1 && slot1.auto_bg) {
				if (slot1.is_main_output) {
					renderOutputSlot(pose, slot1.x, slot1.y);
				} else {
					renderRegularSlot(pose, slot1.x, slot1.y);
				}
			} else if (slot instanceof MatterySlot slot1 && slot1.auto_bg) {
				renderRegularSlot(pose, slot1.x, slot1.y);
			} else if (slot instanceof MatterContainerInputSlot slot1 && slot1.auto_bg) {
				renderRegularSlot(pose, slot1.x, slot1.y);
			}
		}

		for (AbstractWidget widget : menu.widget_list) {
			widget.renderBackground(this, pose, leftPos, topPos, mouseX, mouseY);
		}
	}
}
